"""
Intramuscular EMG simulation components.

This module contains classes and functions for simulating intramuscular EMG signals.
"""

from myogen.simulator.core.emg.intramuscular.intramuscular_emg import IntramuscularEMG
from myogen.simulator.core.emg.intramuscular.motor_unit_sim import MotorUnitSim

__all__ = ["IntramuscularEMG", "MotorUnitSim"]
